@@ -1,11 +1,12 @@
-use crate::direction::Direction;
-use crate::event::{Callback, Event, EventResult, Key, MouseEvent};
-use crate::rect::Rect;
-use crate::theme::{ColorStyle, Effect};
-use crate::utils::lines::simple::{simple_prefix, simple_suffix};
-use crate::view::View;
-use crate::Vec2;
-use crate::{Cursive, Printer, With};
+use crate::{
+    direction::Direction,
+    event::{Callback, Event, EventResult, Key, MouseEvent},
+    rect::Rect,
+    theme::{ColorStyle, Effect},
+    utils::lines::simple::{simple_prefix, simple_suffix},
+    view::{CannotFocus, View},
+    Cursive, Printer, Vec2, With,
+};
 use std::cell::RefCell;
 use std::rc::Rc;
 use unicode_segmentation::UnicodeSegmentation;
@@ -28,7 +29,7 @@ pub type OnSubmit = dyn Fn(&mut Cursive, &str);
 ///
 /// From the [edit example][1].
 ///
-/// [1]: https://github.com/gyscos/cursive/blob/main/examples/src/bin/edit.rs
+/// [1]: https://github.com/gyscos/cursive/blob/main/cursive/examples/edit.rs
 ///
 /// ```rust
 /// # use cursive_core::Cursive;
@@ -148,6 +149,7 @@ impl EditView {
     /// Input will be rejected if it would make the content exceed this width.
     ///
     /// Chainable variant.
+    #[must_use]
     pub fn max_content_width(self, width: usize) -> Self {
         self.with(|s| s.set_max_content_width(Some(width)))
     }
@@ -162,6 +164,7 @@ impl EditView {
     /// Hides the content of the view.
     ///
     /// Only `*` will be shown.
+    #[must_use]
     pub fn secret(self) -> Self {
         self.with(|s| s.set_secret(true))
     }
@@ -183,6 +186,7 @@ impl EditView {
     /// # use cursive_core::views::EditView;
     /// let edit = EditView::new().filler(" ");
     /// ```
+    #[must_use]
     pub fn filler<S: Into<String>>(self, filler: S) -> Self {
         self.with(|s| s.set_filler(filler))
     }
@@ -201,6 +205,7 @@ impl EditView {
     /// When the view is enabled, the style will be reversed.
     ///
     /// Chainable variant.
+    #[must_use]
     pub fn style(self, style: ColorStyle) -> Self {
         self.with(|s| s.set_style(style))
     }
@@ -242,6 +247,7 @@ impl EditView {
     /// Sets a mutable callback to be called whenever the content is modified.
     ///
     /// Chainable variant. See [`set_on_edit_mut`](#method.set_on_edit_mut).
+    #[must_use]
     pub fn on_edit_mut<F>(self, callback: F) -> Self
     where
         F: FnMut(&mut Cursive, &str, usize) + 'static,
@@ -265,6 +271,7 @@ impl EditView {
     ///     content.set_content(format!("{}", text.len()));
     /// });
     /// ```
+    #[must_use]
     pub fn on_edit<F>(self, callback: F) -> Self
     where
         F: Fn(&mut Cursive, &str, usize) + 'static,
@@ -315,6 +322,7 @@ impl EditView {
     /// Sets a mutable callback to be called when `<Enter>` is pressed.
     ///
     /// Chainable variant.
+    #[must_use]
     pub fn on_submit_mut<F>(self, callback: F) -> Self
     where
         F: FnMut(&mut Cursive, &str) + 'static,
@@ -335,6 +343,7 @@ impl EditView {
     ///     s.add_layer(Dialog::info(text));
     /// });
     /// ```
+    #[must_use]
     pub fn on_submit<F>(self, callback: F) -> Self
     where
         F: Fn(&mut Cursive, &str) + 'static,
@@ -369,6 +378,7 @@ impl EditView {
     /// Convenient chainable method.
     ///
     /// Does not run the `on_edit` callback.
+    #[must_use]
     pub fn content<S: Into<String>>(mut self, content: S) -> Self {
         self.set_content(content);
         self
@@ -596,8 +606,11 @@ impl View for EditView {
         self.last_length = size.x;
     }
 
-    fn take_focus(&mut self, _: Direction) -> bool {
-        self.enabled
+    fn take_focus(
+        &mut self,
+        _: Direction,
+    ) -> Result<EventResult, CannotFocus> {
+        self.enabled.then(EventResult::consumed).ok_or(CannotFocus)
     }
 
     fn on_event(&mut self, event: Event) -> EventResult {
