@@ -1,18 +1,15 @@
-use crate::align::{Align, HAlign, VAlign};
-use crate::direction::Direction;
-use crate::event::{
-    Callback, Event, EventResult, Key, MouseButton, MouseEvent,
+use crate::{
+    align::{Align, HAlign, VAlign},
+    direction,
+    event::{Callback, Event, EventResult, Key, MouseButton, MouseEvent},
+    menu,
+    rect::Rect,
+    theme::ColorStyle,
+    utils::markup::StyledString,
+    view::{CannotFocus, Position, View},
+    views::{LayerPosition, MenuPopup},
+    Cursive, Printer, Vec2, With,
 };
-use crate::menu::MenuTree;
-use crate::rect::Rect;
-use crate::theme::ColorStyle;
-use crate::utils::markup::StyledString;
-use crate::view::{Position, View};
-use crate::views::MenuPopup;
-use crate::Cursive;
-use crate::Printer;
-use crate::Vec2;
-use crate::With;
 use std::borrow::Borrow;
 use std::cell::Cell;
 use std::cmp::{min, Ordering};
@@ -117,6 +114,7 @@ impl<T: 'static> SelectView<T> {
     /// item beginning with the pressed letter.
     ///
     /// Chainable variant.
+    #[must_use]
     pub fn autojump(self) -> Self {
         self.with(|s| s.set_autojump(true))
     }
@@ -124,6 +122,7 @@ impl<T: 'static> SelectView<T> {
     /// Turns `self` into a popup select view.
     ///
     /// Chainable variant.
+    #[must_use]
     pub fn popup(self) -> Self {
         self.with(|s| s.set_popup(true))
     }
@@ -148,7 +147,7 @@ impl<T: 'static> SelectView<T> {
     /// # Examples
     ///
     /// ```
-    /// use cursive_core::traits::Identifiable;
+    /// use cursive_core::traits::Nameable;
     /// use cursive_core::views::{SelectView, TextView};
     ///
     /// let text_view = TextView::new("").with_name("text");
@@ -170,6 +169,7 @@ impl<T: 'static> SelectView<T> {
     ///         .unwrap();
     ///     });
     /// ```
+    #[must_use]
     pub fn on_select<F>(self, cb: F) -> Self
     where
         F: Fn(&mut Cursive, &T) + 'static,
@@ -221,6 +221,7 @@ impl<T: 'static> SelectView<T> {
     ///         s.add_layer(Dialog::info(content));
     ///     });
     /// ```
+    #[must_use]
     pub fn on_submit<F, V: ?Sized>(self, cb: F) -> Self
     where
         F: Fn(&mut Cursive, &V) + 'static,
@@ -241,6 +242,7 @@ impl<T: 'static> SelectView<T> {
     ///     .item("One", 1)
     ///     .align(align::Align::top_center());
     /// ```
+    #[must_use]
     pub fn align(mut self, align: Align) -> Self {
         self.align = align;
 
@@ -249,6 +251,7 @@ impl<T: 'static> SelectView<T> {
 
     /// Sets the vertical alignment for this view.
     /// (If the view is given too much space vertically.)
+    #[must_use]
     pub fn v_align(mut self, v: VAlign) -> Self {
         self.align.v = v;
 
@@ -256,6 +259,7 @@ impl<T: 'static> SelectView<T> {
     }
 
     /// Sets the horizontal alignment for this view.
+    #[must_use]
     pub fn h_align(mut self, h: HAlign) -> Self {
         self.align.h = h;
 
@@ -404,6 +408,7 @@ impl<T: 'static> SelectView<T> {
     ///     .item("Item 2", 2)
     ///     .item("Surprise item", 42);
     /// ```
+    #[must_use]
     pub fn item<S: Into<StyledString>>(self, label: S, value: T) -> Self {
         self.with(|s| s.add_item(label, value))
     }
@@ -432,6 +437,7 @@ impl<T: 'static> SelectView<T> {
     /// let select_view = SelectView::new()
     ///     .with_all((1u8..100).into_iter().map(|i| (format!("Item {}", i), i)));
     /// ```
+    #[must_use]
     pub fn with_all<S, I>(self, iter: I) -> Self
     where
         S: Into<StyledString>,
@@ -572,6 +578,7 @@ impl<T: 'static> SelectView<T> {
     /// Chainable variant.
     ///
     /// Does not apply `on_select` callbacks.
+    #[must_use]
     pub fn selected(self, i: usize) -> Self {
         self.with(|s| {
             s.set_selection(i);
@@ -713,7 +720,7 @@ impl<T: 'static> SelectView<T> {
     fn open_popup(&mut self) -> EventResult {
         // Build a shallow menu tree to mimick the items array.
         // TODO: cache it?
-        let mut tree = MenuTree::new();
+        let mut tree = menu::Tree::new();
         for (i, item) in self.items.iter().enumerate() {
             let focus = Rc::clone(&self.focus);
             let on_submit = self.on_submit.as_ref().cloned();
@@ -754,7 +761,10 @@ impl<T: 'static> SelectView<T> {
             // So that we are locked to the parent view.
             // A nice effect is that window resizes will keep both
             // layers together.
-            let current_offset = s.screen().offset();
+            let current_offset = s
+                .screen()
+                .layer_offset(LayerPosition::FromFront(0))
+                .unwrap_or_else(Vec2::zero);
             let offset = offset.signed() - current_offset;
             // And finally, put the view in view!
             s.screen_mut().add_layer_at(
@@ -800,6 +810,7 @@ impl SelectView<String> {
     ///     .item_str("New York")
     ///     .item_str("Tokyo");
     /// ```
+    #[must_use]
     pub fn item_str<S: Into<String>>(self, label: S) -> Self {
         self.with(|s| s.add_item_str(label))
     }
@@ -845,6 +856,7 @@ impl SelectView<String> {
     ///
     /// let select_view = SelectView::new().with_all_str(text.lines());
     /// ```
+    #[must_use]
     pub fn with_all_str<S, I>(self, iter: I) -> Self
     where
         S: Into<String>,
@@ -883,6 +895,7 @@ impl<T: 'static> View for SelectView<T> {
             } else {
                 ColorStyle::primary()
             };
+
             let x = match printer.size.x.checked_sub(1) {
                 Some(x) => x,
                 None => return,
@@ -962,8 +975,28 @@ impl<T: 'static> View for SelectView<T> {
         }
     }
 
-    fn take_focus(&mut self, _: Direction) -> bool {
-        self.enabled && !self.items.is_empty()
+    fn take_focus(
+        &mut self,
+        source: direction::Direction,
+    ) -> Result<EventResult, CannotFocus> {
+        (self.enabled && !self.items.is_empty())
+            .then(|| {
+                if !self.popup {
+                    match source {
+                        direction::Direction::Abs(direction::Absolute::Up) => {
+                            self.focus.set(0);
+                        }
+                        direction::Direction::Abs(
+                            direction::Absolute::Down,
+                        ) => {
+                            self.focus.set(self.items.len().saturating_sub(1));
+                        }
+                        _ => (),
+                    }
+                }
+                EventResult::Consumed(None)
+            })
+            .ok_or(CannotFocus)
     }
 
     fn layout(&mut self, size: Vec2) {
@@ -973,7 +1006,7 @@ impl<T: 'static> View for SelectView<T> {
     fn important_area(&self, size: Vec2) -> Rect {
         self.selected_id()
             .map(|i| Rect::from_size((0, i), (size.x, 1)))
-            .unwrap_or_else(|| Rect::from((0, 0)))
+            .unwrap_or_else(|| Rect::from_size(Vec2::zero(), size))
     }
 }
 
